/*-------------------------------------------------------------------
[SCRIPT]  Afficher les verrous et identifier ce qui est verrouill
[DATABASE] 
[DESCRIPTION] Journes SQL Server 2013
[MAJ PAR] DATAFLY - Arian Papillon
[DATEMAJ]  01/12/2013
-------------------------------------------------------------------*/
GO
USE DB_LOCK
SELECT * FROM INFORMATION_SCHEMA.TABLES
GO
/* 
Traceflags pour voir la pose de verrous
uniquement pour de la dmo !
*/
DBCC TRACEON(3604,1200,-1)
DBCC TRACESTATUS
GO

/* -----------------------------------
Regarder en temps rel les verrous qui sont poss 
-------------------------------------*/

-- SELECT 
SELECT * FROM TABLE1 WHERE T1_ID = 100

-- INSERT
BEGIN TRAN
INSERT TABLE1 (T1_NOM, T1_PRENOM) VALUES ('PAPILLON','Arian')
ROLLBACK

-- UPDATE 
BEGIN TRAN
UPDATE TABLE1 SET T1_PRENOM = 'Jack' WHERE T1_ID = 100

-- Le verrou est pos jusqu'au COMMIT ou ROLLBACK : verrou bloquant potentiel pour une autre transaction
SELECT * FROM sys.dm_tran_locks WHERE resource_database_id = DB_ID('DB_LOCK')  AND request_session_id  = @@spid
-- Dans une autre session
SELECT * FROM TABLE1 WHERE T1_ID = 100

ROLLBACK

-- Verrou SIX
SET TRANSACTION ISOLATION LEVEL SERIALIZABLE
BEGIN TRAN
	-- Un verrou de table est pos
	SELECT * FROM TABLE1 WITH (TABLOCK)
	SELECT * FROM sys.dm_tran_locks WHERE resource_database_id = DB_ID('DB_LOCK')  AND request_session_id  = @@spid
	-- Une ligne de cette table va tre modifi : Verrous sur X sur KEY et IX sur PAGE, et le verrou de table passe en SIX
	UPDATE TABLE1 SET T1_PRENOM = 'Jack' WHERE T1_ID = 100
	SELECT * FROM sys.dm_tran_locks WHERE resource_database_id = DB_ID('DB_LOCK')  AND request_session_id  = @@spid

ROLLBACK
SET TRANSACTION ISOLATION LEVEL READ COMMITTED

-- UPDATE dans une table HEAP
BEGIN TRAN
UPDATE THEAP SET TH_PRENOM = 'Jack' WHERE TH_ID = 100
ROLLBACK

/* Escalade de verrous */
BEGIN TRAN
UPDATE TABLE1 SET T1_PRENOM = 'Jack' WHERE T1_ID < 3000
-- Beaucoup de verrous KEY

SELECT * FROM sys.dm_tran_locks WHERE resource_database_id = DB_ID('DB_LOCK')  AND request_session_id  = @@spid

UPDATE TABLE1 SET T1_PRENOM = 'Jack' WHERE T1_ID < 10000
-- escalade de verrous (+ de 6000 locks pour l'instruction)
-- Un seul verrou sur l'objet
SELECT * FROM sys.dm_tran_locks WHERE resource_database_id = DB_ID('DB_LOCK')  AND request_session_id  = @@spid
ROLLBACK

/* ----------------------------------------------
Comment identifier ce qui est verrouill ? 
-------------------------------------------------*/
---- verrou sur DATABASE 40
SELECT DB_NAME(40) AS Base

---- OBJECT 40:245575913:0
SELECT OBJECT_NAME(245575913,40) AS Object

---- verrou sur PAGE 40:1:126 (base, file_id, page_id)
-- Objet concern
IF object_id('tempdb..#DBCCPageResults') IS NOT NULL DROP TABLE #DBCCPageResults
CREATE TABLE #DBCCPageResults (ParentObject VARCHAR(100), Object VARCHAR(100), Field VARCHAR(100), VALUE VARCHAR(100))
INSERT INTO #DBCCPageResults EXEC ('DBCC PAGE (40,1,126,0) WITH TABLERESULTS')
SELECT Field, Value
FROM #DBCCPageResults 
WHERE Field IN ('Metadata: ObjectId', 'Metadata: IndexId' , 'Metadata: PartitionId', 'Metadata: AllocUnitId')
-- Contenu de la page
SELECT sys.fn_physlocFormatter(%%physloc%%) PhysicalLocation
,%%lockres%% AS LockResHash,TABLE1.*
FROM dbo.TABLE1
CROSS APPLY sys.fn_physlocCracker(%%physloc%%) PL
WHERE PL.file_id = 1 AND PL.page_id=126
-- ou WHERE CAST(CONVERT(BINARY(2),REVERSE(SUBSTRING(%%physloc%%,5, 2))) AS INT) = 1 AND CAST(CONVERT(BINARY(4),REVERSE(SUBSTRING(%%physloc%%,1, 4))) AS INT) = 119

---- KEY 40:72057594039042048 (931e04457546)
-- quelle table et index ?
SELECT 
o.name AS TableName, 
i.name AS IndexName,
SCHEMA_NAME(o.schema_id) AS SchemaName
FROM sys.partitions p JOIN sys.objects o ON p.OBJECT_ID = o.OBJECT_ID 
JOIN sys.indexes i ON p.OBJECT_ID = i.OBJECT_ID  AND p.index_id = i.index_id 
WHERE p.hobt_id = 72057594039042048
-- quelle ligne ?
SELECT %%lockres%% AS LockResHash,* FROM dbo.TABLE1 WHERE %%lockres%% = '(931e04457546)'

---- RID: 40:1:603:99 (base, file_id, page_id, slot_id)
SELECT sys.fn_physlocFormatter(%%physloc%%) PhysicalLocation
,%%lockres%% AS LockResHash,THEAP.*
FROM dbo.THEAP
CROSS APPLY sys.fn_physlocCracker(%%physloc%%) PL
WHERE PL.file_id = 1 AND PL.page_id=603 AND slot_id = 99


-----------------------------
-- Dsactivation des TraceFlags
DBCC TRACEOFF(3604,1200,-1)


